# import mercantile
# import owntwin.builder.utils as utils
# from loguru import logger

id = "owntwin.sample_modules.plateau"

definition = {
    "version": "0.1.0",
    "name": "PLATEAU",
    "description": "国土交通省 Project PLATEAUが提供する3D都市モデルデータです。\n\n<https://www.mlit.go.jp/plateau/>",
    "license": "https://www.mlit.go.jp/plateau/site-policy/",
    "actions": [],
    "layers": [
        {
            "id": "citymodel",
            "name": "3D都市モデル",
            "levels": [0],
            "path": "assets/citymodel.plateau.geojson",
            "format": "geojson",
        }
    ],
}

default_properties = {
    "layers.citymodel.enabled": True,
}


def add(bbox, package, cache_dir):
    pass



import json
from collections import namedtuple
from pathlib import Path
from typing import List

import owntwin.builder.utils as utils
import typer
from geopy.geocoders import Nominatim
from halo import Halo
from InquirerPy import inquirer
from InquirerPy.separator import Separator
from InquirerPy.validator import NumberValidator
from loguru import logger
from owntwin.builder import commands
from owntwin.builder.commands import load_config, save_config
from owntwin.builder.package import Package

app = typer.Typer()

FILENAME = commands.FILENAME


@app.command("init")
def init(dirname: str = typer.Argument(".")):
    dirname = Path(dirname)

    basemap_zoom = commands.BASEMAP_ZOOM  # TODO: Fix

    if Path(dirname.joinpath(FILENAME)).exists():
        typer.Abort()

    if not dirname.is_dir():
        typer.Abort()

    shortdirname = dirname.resolve().name

    uid = inquirer.text(message="Twin ID:", default=shortdirname).execute()

    location_keywords = None
    location = None
    bbox = []
    default_name = ""

    while True:
        location_keywords = inquirer.text(
            message="Location keywords (leave blank to skip):",
        ).execute()

        if not location_keywords:
            break

        geolocator = Nominatim(user_agent="owntwin-cli")
        location_candidates = geolocator.geocode(
            location_keywords, exactly_one=False, limit=5
        )

        if not location_candidates:
            typer.echo("Not found. Please Try again.")
            continue

        location = inquirer.select(
            message="Select nearby location:",
            choices=list(
                map(lambda x: {"name": x.address, "value": x}, location_candidates)
            )
            + [Separator(), {"name": "(Back)", "value": False}],
        ).execute()

        if location:
            break

    if not location_keywords:

        def validate(text):
            try:
                return len(tuple(map(lambda x: float(x.strip()), text.split(",")))) == 2
            except:
                return False

        location_coordinate = inquirer.text(
            message="Location coordinate [latitude, longitude] (leave blank to skip):",
            validate=validate,
            filter=lambda text: tuple(map(lambda x: float(x.strip()), text.split(","))),
        ).execute()
        location = namedtuple("location", ["latitude", "longitude", "address"])
        location.latitude, location.longitude = location_coordinate
        location.address = ""

    if location:
        lat, lng = location.latitude, location.longitude
        # print(location, lat, lng)
        size = inquirer.text(
            message="Side length (meter):",
            filter=lambda x: float(x),
            validate=NumberValidator(float_allowed=True),
        ).execute()

        bbox = utils.bbox_from_center(lat, lng, size)
        bbox = utils.align_bbox(bbox, basemap_zoom)

        typer.echo(
            "  Left (lng): {}\n  Bottom (lat): {}\n  Right (lng): {}\n  Top (lat): {}".format(
                *bbox
            )
        )
        typer.confirm("Is this OK?", default=True, abort=True)
        default_name = location.address

    name = inquirer.text(message="Twin name:", default=default_name).execute()
    type = inquirer.text(message="Twin type:", default="").execute()
    iri = inquirer.text(
        message="IRI or URL:", default=f"https://beta.owntwin.com/twin/{uid}"
    ).execute()
    description = inquirer.text(message="Description:", default="").execute()

    twin = {
        "id": uid,
        "name": name,
        "type": type,
        "type_label": type,
        "iri": iri,
        "description": description,
        "bbox": bbox,
        "terrain": {
            "path": "assets/levelmap.json",
        },
        "building": {
            "path": "assets/buildings.json",
        },
        "properties": {},
        "modules": {},
    }

    # twin = {
    #     "name": name,
    #     ...
    #     "terrain": "levelmap.json",
    #     "objects": {
    #       "building": "building.json",
    #     },
    #     "modules": {},
    # }

    typer.echo(f"About to create {FILENAME}:\n")
    typer.echo(json.dumps(twin, ensure_ascii=False, indent=2))
    typer.confirm("Is this OK?", default=True, abort=True)

    if not dirname.exists():
        dirname.mkdir()

    save_config(twin, dirname.joinpath(FILENAME))


@app.command("add-terrain")
def add_terrain():
    twin = load_config()

    bbox = twin["bbox"]
    package = Package(".")

    with Halo(
        text="Installing {}".format(
            typer.style("terrain", fg=typer.colors.GREEN, bold=True)
        ),
        spinner="bouncingBar",
    ) as spinner:
        commands.add_terrain(bbox, package)
        spinner.succeed()


@app.command("add")
def add(module_names: List[str]):
    # NOTE: a module name can be:
    # local file, PyPI name, git repo/GitHub?

    twin = load_config()

    spinner = Halo(text="", spinner="bouncingBar")
    for module_name in module_names:
        module_name = commands.resolve_module_name(module_name)
        try:
            spinner.start(
                "Installing {}".format(
                    typer.style(
                        module_name,
                        fg=typer.colors.GREEN,
                        bold=True,
                    )
                )
            )
            commands.add_module(twin, Package("."), module_name)
            spinner.succeed()
        except Exception as err:
            logger.error(err)
            spinner.fail()

    spinner.stop()


@app.command("export")
def export(dirname: str = typer.Argument(".")):
    with Halo(text="Exporting...", spinner="bouncingBar") as spinner:
        commands.export(dirname)
        spinner.succeed(text="Done!")


@app.callback()
def main():
    pass


if __name__ == "__main__":
    app()



import io

from loguru import logger
import svgwrite
from scour import scour
import pyproj

NONE = "none"
WHITE = "#FFFFFF"
GRAY = "#969696"


def render(
    gdf,
    outfile,
    bbox=None,
    filter=None,
    optimize=True,
    fill=NONE,
    stroke=GRAY,
    stroke_width=1,
):
    gdf = gdf.copy()
    logger.debug(gdf.crs)
    gdf = gdf.to_crs(epsg=3857)  # 3395

    logger.debug(gdf.head())

    # logger.debug(gdf.columns)
    # for v in gdf[['class', 'type', 'name']].iterrows():
    #     logger.debug(v)
    # exit()

    if bbox is None:
        bbox = gdf.geometry.total_bounds
    else:
        transformer = pyproj.Transformer.from_crs(
            "epsg:4326", "epsg:3857", always_xy=True
        )
        ul = transformer.transform(bbox[0], bbox[1])
        br = transformer.transform(bbox[2], bbox[3])
        bbox = [*ul, *br]
        # logger.debug(f"\n{bbox}\n{gdf.geometry.total_bounds}")

    minx, miny, maxx, maxy = bbox
    gdf.geometry = gdf.geometry.translate(-minx, -miny)
    maxx, maxy = maxx - minx, maxy - miny

    # gdf = gdf.scale(100000, 100000, origin=(0, 0))

    viewbox = " ".join(map(str, [0, 0, maxx, maxy]))
    dwg = svgwrite.Drawing(outfile, height="100%", width="100%", viewBox=(viewbox))

    # NOTE: Not used in SVGMeshLayer (path only)
    dwg.fill(color=fill)
    dwg.stroke(color=stroke, width=stroke_width)
    extras = {}

    if filter is not None:
        filtered_gdf = gdf[filter]
    else:
        filtered_gdf = gdf

    for g in filtered_gdf.geometry:
        # TODO: Reconsider precision
        mp = [(round(x, 5), round(maxy - y, 5)) for x, y in zip(*g.coords.xy)]

        # gr = svgwrite.container.Group(**extras)
        dp = dwg.polyline(points=mp)
        # gr.add(dp)
        dwg.add(dp)

    if optimize:

        class ScourOptions:
            quiet = True

        buf1 = io.StringIO()
        dwg.write(buf1)
        buf1.seek(0)
        buf1 = io.BytesIO(buf1.getvalue().encode())
        buf1.seek(0)
        buf1.name = outfile
        with open(outfile, "wb") as f:
            scour.start(ScourOptions, buf1, f)
    else:
        with open(outfile, "w", encoding="utf-8") as f:
            dwg.write(f)


def render_full(gdf, bbox=None, outfile="full.svg", **kwargs):
    render(gdf, outfile, bbox=bbox, **kwargs)


def render_contour(gdf, bbox=None, outfile="contour.svg", **kwargs):
    render(gdf, outfile, bbox=bbox, filter=(gdf["class"] == "Cntr"), **kwargs)



import os
from pathlib import Path

from xdg import xdg_cache_home

CACHE_DIR = None

if os.name == "nt":
    CACHE_DIR = Path(os.path.expandvars("%APPDATA%/owntwin/cache"))
else:
    CACHE_DIR = xdg_cache_home().joinpath("owntwin/")

if not CACHE_DIR.exists():
    CACHE_DIR.mkdir(parents=True)



id = "owntwin.sample_modules.jpsearch"

definition = {
    "version": "0.1.0",
    "name": "ジャパンサーチ",
    "description": None,
    "license": "https://jpsearch.go.jp/policy",
    "actions": [
        {
            "id": "search_by_name",
            "type": "link",
            "text": "場所名でジャパンサーチを検索",
            "href": "https://jpsearch.go.jp/csearch/jps-cross",
            "default_param": "csid=jps-cross",
            "fields": ["name"],
            "fields.assign_to": {"name": "q-loc"},
        },
        {
            "id": "search_by_location",
            "type": "link",
            "text": "位置情報でジャパンサーチを検索",
            "href": "https://beta.owntwin.com/resolver/jpsearch",
            "fields": ["owntwin:geohash"],
            "fields.assign_to": {"owntwin:geohash": "geohash"},
        },
    ],
    "layers": [],
}

default_properties = {}


def add(bbox, package, cache_dir):
    pass



import mercantile
import owntwin.builder.utils as utils
from loguru import logger
from owntwin.builder.render import render_full
from owntwin.builtin_datasources import gsi

id = "owntwin.base"

definition = {
    "version": "0.1.0",
    "name": "基本",
    "description": """国土地理院ベクトルタイル提供実験による「基盤地図情報_基本項目」に基づくベースマップ等を含む基本モジュールです。

<https://github.com/gsi-cyberjapan/vector-tile-experiment>""",
    "license": "https://github.com/gsi-cyberjapan/vector-tile-experiment#%E6%8F%90%E4%BE%9B%E3%81%AE%E4%BD%8D%E7%BD%AE%E3%81%A5%E3%81%91",
    "actions": [
        {
            "id": "suggestion",
            "type": "link",
            "text": "この場所の情報修正を提案",
            "fields": ["name", "iri"],
        }
    ],
    "layers": [
        {
            "id": "basemap",
            "name": "ベースマップ",
            "levels": [0],
            "path": "assets/basemap.svg",
            "format": "svg",
            "color": "rgb(229, 231, 235)",
        },
        {
            "id": "rdcl",
            "name": "道路",
            "levels": [0],
            "path": "assets/rdcl.svg",
            "format": "svg",
            "color": "rgb(209, 213, 219)",
        },
        # {
        #     "id": "rdcl",
        #     "name": "道路",
        #     "levels": [0],
        #     "path": "assets/rdcl.geojson",
        #     "format": "geojson",
        #     "colors": {"default": "rgb(255, 255, 255)"},
        # },
        {
            "id": "railcl",
            "name": "鉄道",
            "levels": [0],
            "path": "assets/railcl.svg",
            "format": "svg",
            "color": "rgb(14, 116, 144)",
        },
        {
            "id": "rvcl",
            "name": "河川",
            "levels": [0],
            "path": "assets/rvcl.svg",
            "format": "svg",
            "color": "rgb(59, 130, 246)",
        },
    ],
}

default_properties = {
    "actions.suggestion.href": None,
    "actions.suggestion.fields.assign_to": {
        "name": None,
        "iri": None,
    },
    "layers.basemap.enabled": True,
    "layers.rdcl.enabled": True,
    "layers.railcl.enabled": True,
    "layers.rvcl.enabled": True,
}


def bbox_to_tiles(bbox, zoom):
    tiles = mercantile.tiles(*bbox, zoom)  # left, bottom, right, top
    tiles = list(tiles)
    # print(tiles)
    # ul = mercantile.ul(tiles[0])
    # br = [mercantile.bounds(tiles[-1]).east, mercantile.bounds(tiles[-1]).south]
    # basemap_bbox = [*ul, *br]
    return tiles


def add(bbox, package, cache_dir):
    dl = gsi.Downloader(cache_dir)

    ### basemap (fgd)
    BASEMAP_ZOOM = 18
    tiles = bbox_to_tiles(bbox, BASEMAP_ZOOM)
    basemap_bbox = utils.tiles_bounds(tiles)
    logger.debug(("basemap_bbox", basemap_bbox))

    filenames = dl.download_fgd(tiles)

    # filenames = []
    # for filename in Path(dl.cwd).glob("fgd-18*.geojson"):
    #     filenames.append(filename)

    merged = utils.geojson_merge(filenames)
    logger.debug(merged.head())

    if not merged.empty:  # TODO: Handle empty DataFrame
        merged.to_file(package.assets.joinpath("fgd.geojson"), driver="GeoJSON")
    render_full(
        merged, bbox=basemap_bbox, outfile=package.assets.joinpath("basemap.svg")
    )

    ### roadmap (rdcl)
    RDCL_ZOOM = 16  # TODO: Must be consistent with BASEMAP_ZOOM
    tiles = bbox_to_tiles(bbox, RDCL_ZOOM)
    filenames = dl.download_rdcl(tiles)

    merged = utils.geojson_merge(filenames)
    logger.debug(merged.head())

    if not merged.empty:
        merged.to_file(package.assets.joinpath("rdcl.geojson"), driver="GeoJSON")
    # TODO: basemap_bbox or else
    render_full(merged, bbox=basemap_bbox, outfile=package.assets.joinpath("rdcl.svg"))

    ### railmap (railcl)
    RAILCL_ZOOM = 16  # TODO: Must be consistent with BASEMAP_ZOOM
    tiles = bbox_to_tiles(bbox, RAILCL_ZOOM)
    filenames = dl.download_railcl(tiles)

    merged = utils.geojson_merge(filenames)
    logger.debug(merged.head())

    if not merged.empty:
        merged.to_file(package.assets.joinpath("railcl.geojson"), driver="GeoJSON")
    # TODO: basemap_bbox or else
    render_full(
        merged, bbox=basemap_bbox, outfile=package.assets.joinpath("railcl.svg")
    )

    ### rivermap (rvcl)
    RVCL_ZOOM = 16
    tiles = bbox_to_tiles(bbox, RVCL_ZOOM)
    filenames = dl.download_rvcl(tiles)

    merged = utils.geojson_merge(filenames)
    logger.debug(merged.head())

    if not merged.empty:
        merged.to_file(package.assets.joinpath("rvcl.geojson"), driver="GeoJSON")
    render_full(merged, bbox=basemap_bbox, outfile=package.assets.joinpath("rvcl.svg"))



import codecs
import json
import shutil
from importlib import import_module
from pathlib import Path
from typing import List

# import importlib.resources  # Strangely not working
import importlib_resources  # Should be replaced with importlib.resource
import mercantile
import owntwin.builder.utils as utils
from loguru import logger
from owntwin.builder import CACHE_DIR
from owntwin.builder.terrain import extract_meshed_level
from owntwin.builtin_datasources import gsi

FILENAME = "twin.json"
BASEMAP_ZOOM = 18  # TODO: Fix
DEM5A_ZOOM = 18

def load_config():
    with open(FILENAME, "r") as f:
        twin = json.load(f)
    return twin


def save_config(config, path):
    # NOTE: Use codecs.open for win
    with codecs.open(path, "w", "utf-8") as f:
        json.dump(config, f, ensure_ascii=False, indent=2)


def add_terrain(bbox: List[float], package):
    dl = gsi.Downloader(CACHE_DIR)  # TODO: Fix

    tiles = mercantile.tiles(*bbox, DEM5A_ZOOM)  # left, bottom, right, top
    tiles = list(tiles)

    filenames = dl.download_dem5a(tiles)
    merged = utils.geojson_merge(filenames)
    # print(merged.head())

    merged.to_file(package.assets.joinpath("merged_dem5a.geojson"), driver="GeoJSON")

    extract_meshed_level(merged, outfile=package.assets.joinpath("levelmap.json"))


def resolve_module_name(module_name: str):
    builtin_modules_map = {
        "owntwin.base": "owntwin.builtin_modules.base",
    }

    module_name = builtin_modules_map.get(module_name, module_name)
    return module_name


def add_module(twin, package, module_name: str):
    module = import_module(module_name)  # NOTE: module_name must be resolved

    module_append = (module.id, module.definition, module.default_properties)
    module.add(twin["bbox"], package, CACHE_DIR)

    (module_name, definition, default_properties) = module_append

    if not twin["modules"].get(module_name, None):
        twin["modules"][module_name] = {}

    twin["modules"][module_name]["version"] = "^{}".format(
        definition["version"]
    )  # TODO: Fix

    if not twin.get("properties", None):
        twin["properties"] = {}

    for key, value in default_properties.items():
        ns_key = "{}:{}".format(module_name, key)
        if not twin["properties"].get(ns_key, None):
            twin["properties"][ns_key] = value

    twin["modules"][module_name]["definition"] = definition

    save_config(twin, package.cwd.joinpath(FILENAME))  # TODO: Do not save every time

    return twin


def export(dirname: str):
    # https://importlib-resources.readthedocs.io/en/latest/migration.html
    path = importlib_resources.files("owntwin.viewer.owntwin")
    for entry in path.iterdir():
        logger.info(entry)
        if entry.is_dir():
            shutil.copytree(
                entry,
                Path(dirname).joinpath(entry.name),
                dirs_exist_ok=True,  # TODO: Confirm
                copy_function=shutil.copy,
                ignore=shutil.ignore_patterns(".DS_Store"),
            )
        else:
            if not any(
                [
                    entry.match(exclude)
                    for exclude in ["twin.json", "assets", ".DS_Store"]
                ]
            ):
                shutil.copy(entry, dirname)



import os
import posixpath
import re
import sys
import urllib
from functools import partial
from pathlib import Path

import importlib.resources
import typer
from loguru import logger

import owntwin.builder.cli

logger.remove()
logger.add(sys.stderr, format="{message}", level="ERROR", backtrace=True, diagnose=True)
# logger.add(sys.stderr, format="{message}", level="INFO")

app = typer.Typer()

# app.add_typer(owntwin.builder.cli.app, name="builder")

app.command("init")(owntwin.builder.cli.init)
app.command("add-terrain")(owntwin.builder.cli.add_terrain)
app.command("add")(owntwin.builder.cli.add)
app.command("export")(owntwin.builder.cli.export)


@app.command("view")
def view(
    dirname: str = typer.Argument("."),
    port: int = typer.Option(8000),
    no_terrain: bool = typer.Option(False, "--no-terrain", help="Disable terrain"),
):
    import socketserver
    from http.server import SimpleHTTPRequestHandler

    class RequestHandler(SimpleHTTPRequestHandler):
        def log_message(self, format, *args):
            pass

        def end_headers(self):
            self.send_header("Access-Control-Allow-Origin", "*")
            self.send_header("Cache-Control", "no-cache, no-store, must-revalidate")
            self.send_header("Pragma", "no-cache")
            self.send_header("Expires", "0")
            SimpleHTTPRequestHandler.end_headers(self)

        def translate_path(self, path):
            if path.startswith("/__/"):
                path = re.sub(r"^/__/", "/", path)
                directory = str(Path(dirname))
            else:
                directory = self.directory
            # print(path, directory)
            # abandon query parameters
            path = path.split("?", 1)[0]
            path = path.split("#", 1)[0]
            # Don't forget explicit trailing slash when normalizing. Issue17324
            trailing_slash = path.rstrip().endswith("/")
            try:
                path = urllib.parse.unquote(path, errors="surrogatepass")
            except UnicodeDecodeError:
                path = urllib.parse.unquote(path)
            path = posixpath.normpath(path)
            words = path.split("/")
            words = filter(None, words)
            path = directory
            for word in words:
                if os.path.dirname(word) or word in (os.curdir, os.pardir):
                    # Ignore components that are not a simple file/directory name
                    continue
                path = os.path.join(path, word)
            if trailing_slash:
                path += "/"
            return path

    viewer_dir = importlib.resources.files("owntwin.viewer.owntwin").joinpath("")

    Handler = partial(
        RequestHandler,
        # directory=Path(__file__).parent.joinpath("./viewer/owntwin/"),
        directory=viewer_dir,
    )

    Server = socketserver.TCPServer
    Server.allow_reuse_address = True

    with Server(("localhost", port), Handler) as httpd:
        print(
            f"Started preview on http://localhost:{port}/?twin=http://localhost:{port}/__/{'&no-terrain' if no_terrain else ''}"
        )
        httpd.serve_forever()


@app.callback()
def main(verbose: int = typer.Option(0, "--verbose", "-v", count=True)):
    if verbose == 1:
        logger.remove()
        logger.add(
            sys.stderr, format="{message}", level="INFO", backtrace=True, diagnose=True
        )
    elif verbose == 2 or verbose > 2:
        logger.remove()
        logger.add(
            sys.stderr, format="{message}", level="DEBUG", backtrace=True, diagnose=True
        )


if __name__ == "__main__":
    app()



import os
from io import BytesIO
from pathlib import Path
from time import sleep
from urllib.parse import urlparse

import mercantile
import owntwin.builder.utils as utils
import pandas as pd

# from owntwin.builtin_datasources import gsi_disaportal
import requests
from loguru import logger
from owntwin.builder import CACHE_DIR

id = "owntwin.sample_modules.bousai_tokyo"

TOKYO_BM_HINANJO_URL = (
    "https://www.opendata.metro.tokyo.lg.jp/soumu/130001_evacuation_center.csv"
)
TOKYO_BM_HINANBASHO_URL = (
    "https://www.opendata.metro.tokyo.lg.jp/soumu/130001_evacuation_area.csv"
)
TOKYO_BM_FIREHYDRANT_URL = (
    "https://www.opendata.metro.tokyo.lg.jp/shoubou/2021/130001_fire-hydrant.csv"
)
TOKYO_BM_WATERTANK_URL = "https://www.opendata.metro.tokyo.lg.jp/shoubou/2021/130001_fireproof-water-tank.csv"

SUIDOU_WATERSUPPLY_URL = "https://www.opendata.metro.tokyo.lg.jp/suidou/R3/kyoten.csv"

HOKEN_DAREDEMOWC_KOKYO_URL = "https://www.opendata.metro.tokyo.lg.jp/fukushihoken/wc/koukyoshisetsu_barrier-free-wc.csv"
HOKEN_DAREDEMOWC_STATION_URL = "https://www.opendata.metro.tokyo.lg.jp/fukushihoken/wc/tonaitetsudoueki_barrier-free-wc.csv"
HOKEN_SHINRYOUKENSA_URL = "https://www.opendata.metro.tokyo.lg.jp/fukushihoken/130001_shinryoukensa20220112.csv"

definition = {
    "version": "0.1.0",
    "name": "防災・災害対応（東京都）",
    "description": "東京都オープンデータカタログ掲載のオープンデータです。",
    "license": "避難所、避難場所データ オープンデータ (CC-BY-4.0) <https://catalog.data.metro.tokyo.lg.jp/dataset/t000003d0000000093>\n"
    + "東京消防庁 消火栓及び防火水槽等 (CC-BY-4.0) <https://catalog.data.metro.tokyo.lg.jp/dataset/t000017d0000000007>\n"
    + "東京都水道局 給水拠点一覧データ (CC-BY-4.0) <https://catalog.data.metro.tokyo.lg.jp/dataset/t000019d0000000001>\n"
    + "東京都福祉保健局 だれでもトイレのバリアフリー情報 (CC-BY-4.0) <https://catalog.data.metro.tokyo.lg.jp/dataset/t000010d0000000062>\n"
    + "東京都福祉保健局  診療・検査医療機関の一覧  (CC-BY-4.0) <https://catalog.data.metro.tokyo.lg.jp/dataset/t000010d0000000095>",
    "actions": [
        {
            "id": "manual",
            "type": "link",
            "text": "この場所の防災マニュアルを見る",
            "href": None,
            "fields": [],
        },
        # {
        #     "id": "anpi",
        #     "type": "link",
        #     "text": "この場所から安否状況を登録（発災時のみ）",
        #     "href": None,
        #     "fields": ["name", "iri"],
        # },
    ],
    "layers": [
        {
            "id": "tokyo_bm_hinanjo",
            "name": "避難所",
            "path": "assets/tokyo_bm_hinanjo.csv",
            "format": "csv",
            "labelVisibility": "always",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": "避難所_名称",
            },
        },
        {
            "id": "tokyo_bm_hinanbasho",
            "name": "避難場所",
            "path": "assets/tokyo_bm_hinanbasho.csv",
            "format": "csv",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": "避難場所_名称",
            },
        },
        {
            "id": "tokyo_bm_firehydrant",
            "name": "消火栓",
            "path": "assets/tokyo_bm_firehydrant.csv",
            "format": "csv",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": None,
            },
            "color": 0xFA913C,
            "size": {"height": 16},
        },
        {
            "id": "tokyo_bm_watertank",
            "name": "防火水槽等",
            "path": "assets/tokyo_bm_watertank.csv",
            "format": "csv",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": None,
            },
            "color": 0x6366F1,
            "size": {"height": 16},
        },
        {
            "id": "suidou_watersupply",
            "name": "給水拠点",
            "path": "assets/suidou_watersupply.csv",
            "format": "csv",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": "施設名",
            },
            "color": 0x6366F1,
            "size": {"height": 16},
        },
        {
            "id": "hoken_daredemowc",
            "name": "だれでもトイレ（バリアフリー）",
            "path": "assets/hoken_daredemowc.csv",
            "format": "csv",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": "トイレ名",
            },
            "color": 0x06B6D4,
            "size": {"height": 32},
        },
        {
            "id": "hoken_shinryoukensa",
            "name": "診療・検査医療機関",
            "path": "assets/hoken_shinryoukensa.csv",
            "format": "csv",
            "keys": {
                "lat": "緯度",
                "lng": "経度",
                "label": "医療機関名",
            },
            "color": 0xE879F9,
            "size": {"height": 50},
        },
    ],
}

default_properties = {
    "actions.manual.href": None,
    "actions.anpi.href": None,
    "actions.anpi.fields.assign_to": {
        "name": None,
        "iri": None,
    },
    "layers.tokyo_bm_hinanjo.enabled": True,
    "layers.tokyo_bm_hinanbasho.enabled": True,
    "layers.tokyo_bm_firehydrant.enabled": False,
    "layers.tokyo_bm_watertank.enabled": True,
    "layers.suidou_watersupply.enabled": True,
    "layers.hoken_daredemowc.enabled": False,
    "layers.hoken_shinryoukensa.enabled": True,
}


def save_areadata(bbox, url_or_urls, filename, encoding=None, cache=True):
    if not isinstance(url_or_urls, (list, tuple)):
        urls = [url_or_urls]
    else:
        urls = url_or_urls

    area_df = pd.DataFrame()

    for url in urls:
        cachefile = Path(CACHE_DIR) / os.path.basename(urlparse(url).path)
        data = None
        if cache and cachefile.exists():
            with open(cachefile, "rb") as f:
                data = BytesIO(f.read())
        if not cache or not data:
            resp = requests.get(url)
            # if resp.status_code == 404:
            #     raise
            logger.debug(("resp.apparent_encoding", resp.apparent_encoding))
            encoding = encoding or resp.apparent_encoding
            content = resp.content.decode(encoding)
            data = BytesIO(content.encode("utf-8"))
            if cache:
                with open(cachefile, "wb") as f:
                    f.write(data.read())
                data.seek(0)

        df = pd.read_csv(data, sep=",")
        logger.debug(df.columns)
        # NOTE: NaN in bad rows
        df[["緯度", "経度"]] = df[["緯度", "経度"]].apply(pd.to_numeric, errors="coerce")
        # df[["緯度", "経度"]] = df[["緯度", "経度"]].astype("float")
        area_df = area_df.append(
            df.query("@bbox[1] <= 緯度 <= @bbox[3] and @bbox[0] <= 経度 <= @bbox[2]")
        )
        logger.debug(("area_df", area_df))

    area_df.to_csv(filename, encoding="utf-8", sep=",")

    sleep(1)


def add(bbox, package, cache_dir):
    basemap_zoom = 18
    tiles = mercantile.tiles(*bbox, basemap_zoom)  # left, bottom, right, top
    tiles = list(tiles)
    basemap_bbox = utils.tiles_bounds(tiles)
    logger.debug(("basemap_bbox", basemap_bbox))

    save_areadata(
        basemap_bbox,
        TOKYO_BM_HINANJO_URL,
        package.assets.joinpath("tokyo_bm_hinanjo.csv"),
    )
    save_areadata(
        basemap_bbox,
        TOKYO_BM_HINANBASHO_URL,
        package.assets.joinpath("tokyo_bm_hinanbasho.csv"),
    )
    save_areadata(
        basemap_bbox,
        TOKYO_BM_FIREHYDRANT_URL,
        package.assets.joinpath("tokyo_bm_firehydrant.csv"),
    )
    save_areadata(
        basemap_bbox,
        TOKYO_BM_WATERTANK_URL,
        package.assets.joinpath("tokyo_bm_watertank.csv"),
    )

    save_areadata(
        basemap_bbox,
        SUIDOU_WATERSUPPLY_URL,
        package.assets.joinpath("suidou_watersupply.csv"),
    )

    save_areadata(
        basemap_bbox,
        [HOKEN_DAREDEMOWC_KOKYO_URL, HOKEN_DAREDEMOWC_STATION_URL],
        package.assets.joinpath("hoken_daredemowc.csv"),
        encoding="cp932",
    )

    save_areadata(
        basemap_bbox,
        [HOKEN_SHINRYOUKENSA_URL],
        package.assets.joinpath("hoken_shinryoukensa.csv"),
    )
